@@ -3,43 +3,44 @@
     SPDX-FileCopyrightText: 1999-2000 Waldo Bastian <bastian@kde.org>
     SPDX-FileCopyrightText: 2005-2009 David Faure <faure@kde.org>
     SPDX-FileCopyrightText: 2008 Hamish Rodda <rodda@kde.org>
-    SPDX-FileCopyrightText: 2020 Harald Sitter <sitter@kde.org>
+    SPDX-FileCopyrightText: 2020-2022 Harald Sitter <sitter@kde.org>
 
     SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "ksycoca.h"
 #include "ksycoca_p.h"
-#include "ksycocautils_p.h"
-#include "ksycocatype.h"
 #include "ksycocafactory_p.h"
-#include <KConfigGroup>
-#include <KSharedConfig>
+#include "ksycocatype.h"
+#include "ksycocautils_p.h"
 #include "sycocadebug.h"
+#include <KConfigGroup>
+#include <KSandbox>
+#include <KSharedConfig>
 
-#include <QStandardPaths>
-#include <QDataStream>
 #include <QCoreApplication>
+#include <QDataStream>
 #include <QFile>
 #include <QFileInfo>
+#include <QMetaMethod>
+#include <QStandardPaths>
 #include <QThread>
 #include <QThreadStorage>
-#include <QMetaMethod>
 
-#include <stdlib.h>
+#include <QCryptographicHash>
 #include <fcntl.h>
 #include <kmimetypefactory_p.h>
-#include <kservicetypefactory_p.h>
-#include <kservicegroupfactory_p.h>
 #include <kservicefactory_p.h>
-#include <QCryptographicHash>
+#include <kservicegroupfactory_p.h>
+#include <kservicetypefactory_p.h>
+#include <stdlib.h>
 
 #include "kbuildsycoca_p.h"
 #include "ksycocadevices_p.h"
 
 #ifdef Q_OS_UNIX
-#include <utime.h>
 #include <sys/time.h>
+#include <utime.h>
 #endif
 
 /**
@@ -56,10 +57,11 @@
 #endif
 
 #ifndef MAP_FAILED
-#define MAP_FAILED ((void *) -1)
+#define MAP_FAILED ((void *)-1)
 #endif
 
-QDataStream &operator>>(QDataStream &in, KSycocaHeader &h) {
+QDataStream &operator>>(QDataStream &in, KSycocaHeader &h)
+{
     in >> h.prefixes >> h.timeStamp >> h.language >> h.updateSignature;
     return in;
 }
@@ -74,23 +76,23 @@ QDataStream &operator>>(QDataStream &in, KSycocaHeader &h) {
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(KSycocaPrivate::BehaviorsIfNotFound)
 
-KSycocaPrivate::KSycocaPrivate(KSycoca *q)
-    : databaseStatus(DatabaseNotOpen),
-      readError(false),
-      timeStamp(0),
-      m_databasePath(),
-      updateSig(0),
-      m_fileWatcher(new KDirWatch),
-      m_haveListeners(false),
-      q(q),
-      sycoca_size(0),
-      sycoca_mmap(nullptr),
-      m_mmapFile(nullptr),
-      m_device(nullptr),
-      m_mimeTypeFactory(nullptr),
-      m_serviceTypeFactory(nullptr),
-      m_serviceFactory(nullptr),
-      m_serviceGroupFactory(nullptr)
+KSycocaPrivate::KSycocaPrivate(KSycoca *qq)
+    : databaseStatus(DatabaseNotOpen)
+    , readError(false)
+    , timeStamp(0)
+    , m_databasePath()
+    , updateSig(0)
+    , m_fileWatcher(new KDirWatch)
+    , m_haveListeners(false)
+    , q(qq)
+    , sycoca_size(0)
+    , sycoca_mmap(nullptr)
+    , m_mmapFile(nullptr)
+    , m_device(nullptr)
+    , m_mimeTypeFactory(nullptr)
+    , m_serviceTypeFactory(nullptr)
+    , m_serviceFactory(nullptr)
+    , m_serviceGroupFactory(nullptr)
 {
 #ifdef Q_OS_WIN
     /*
@@ -130,9 +132,7 @@ bool KSycocaPrivate::tryMmap()
     }
     fcntl(m_mmapFile->handle(), F_SETFD, FD_CLOEXEC);
     sycoca_size = m_mmapFile->size();
-    void *mmapRet = mmap(nullptr, sycoca_size,
-                       PROT_READ, MAP_SHARED,
-                       m_mmapFile->handle(), 0);
+    void *mmapRet = mmap(nullptr, sycoca_size, PROT_READ, MAP_SHARED, m_mmapFile->handle(), 0);
     /* POSIX mandates only MAP_FAILED, but we are paranoid so check for
        null pointer too.  */
     if (mmapRet == MAP_FAILED || mmapRet == nullptr) {
@@ -142,7 +142,7 @@ bool KSycocaPrivate::tryMmap()
     } else {
         sycoca_mmap = static_cast<const char *>(mmapRet);
 #if HAVE_MADVISE
-        (void) posix_madvise(mmapRet, sycoca_size, POSIX_MADV_WILLNEED);
+        (void)posix_madvise(mmapRet, sycoca_size, POSIX_MADV_WILLNEED);
 #endif // HAVE_MADVISE
         return true;
     }
@@ -159,8 +159,12 @@ int KSycoca::version()
 class KSycocaSingleton
 {
 public:
-    KSycocaSingleton() { }
-    ~KSycocaSingleton() { }
+    KSycocaSingleton()
+    {
+    }
+    ~KSycocaSingleton()
+    {
+    }
 
     bool hasSycoca() const
     {
@@ -210,9 +214,13 @@ KSycoca::KSycoca()
 {
     if (d->m_fileWatcher) {
         // We always delete and recreate the DB, so KDirWatch normally emits created
-        connect(d->m_fileWatcher.get(), &KDirWatch::created, this, [this](){ d->slotDatabaseChanged(); });
+        connect(d->m_fileWatcher.get(), &KDirWatch::created, this, [this]() {
+            d->slotDatabaseChanged();
+        });
         // In some cases, KDirWatch only thinks the file was modified though
-        connect(d->m_fileWatcher.get(), &KDirWatch::dirty, this, [this]() { d->slotDatabaseChanged(); });
+        connect(d->m_fileWatcher.get(), &KDirWatch::dirty, this, [this]() {
+            d->slotDatabaseChanged();
+        });
     }
 }
 
@@ -220,7 +228,8 @@ bool KSycocaPrivate::openDatabase()
 {
     Q_ASSERT(databaseStatus == DatabaseNotOpen);
 
-    delete m_device; m_device = nullptr;
+    delete m_device;
+    m_device = nullptr;
 
     if (m_databasePath.isEmpty()) {
         m_databasePath = findDatabase();
@@ -228,11 +237,22 @@ bool KSycocaPrivate::openDatabase()
 
     bool result = true;
     if (!m_databasePath.isEmpty()) {
+        static bool firstTime = true;
+        if (firstTime) {
+            firstTime = false;
+            if (KSandbox::isFlatpak()) {
+                // We're running inside flatpak, which sets all times to 1970
+                // So the first very time, don't use an existing database, recreate it
+                qCDebug(SYCOCA) << "flatpak detected, ignoring" << m_databasePath;
+                return false;
+            }
+        }
+
         qCDebug(SYCOCA) << "Opening ksycoca from" << m_databasePath;
         m_dbLastModified = QFileInfo(m_databasePath).lastModified();
         result = checkVersion();
     } else { // No database file
-        //qCDebug(SYCOCA) << "Could not open ksycoca";
+        // qCDebug(SYCOCA) << "Could not open ksycoca";
         result = false;
     }
     return result;
@@ -248,10 +268,10 @@ KSycocaAbstractDevice *KSycocaPrivate::device()
     Q_ASSERT(!m_databasePath.isEmpty());
 #if HAVE_MMAP
     if (m_sycocaStrategy == StrategyMmap && tryMmap()) {
-        device = new KSycocaMmapDevice(sycoca_mmap,
-                sycoca_size);
+        device = new KSycocaMmapDevice(sycoca_mmap, sycoca_size);
         if (!device->device()->open(QIODevice::ReadOnly)) {
-            delete device; device = nullptr;
+            delete device;
+            device = nullptr;
         }
     }
 #endif
@@ -259,7 +279,8 @@ KSycocaAbstractDevice *KSycocaPrivate::device()
     if (!device && m_sycocaStrategy == StrategyMemFile) {
         device = new KSycocaMemFileDevice(m_databasePath);
         if (!device->device()->open(QIODevice::ReadOnly)) {
-            delete device; device = nullptr;
+            delete device;
+            device = nullptr;
         }
     }
 #endif
@@ -267,7 +288,7 @@ KSycocaAbstractDevice *KSycocaPrivate::device()
         device = new KSycocaFileDevice(m_databasePath);
         if (!device->device()->open(QIODevice::ReadOnly)) {
             qCWarning(SYCOCA) << "Couldn't open" << m_databasePath << "even though it is readable? Impossible.";
-            //delete device; device = 0; // this would crash in the return statement...
+            // delete device; device = 0; // this would crash in the return statement...
         }
     }
     if (device) {
@@ -304,7 +325,10 @@ void KSycocaPrivate::slotDatabaseChanged()
 
     // Now notify applications
     Q_EMIT q->databaseChanged();
+
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 80)
     Q_EMIT q->databaseChanged(changeList);
+#endif
 }
 
 KMimeTypeFactory *KSycocaPrivate::mimeTypeFactory()
@@ -363,7 +387,7 @@ KSycoca::~KSycoca()
 {
     d->closeDatabase();
     delete d;
-    //if (ksycocaInstance.exists()
+    // if (ksycocaInstance.exists()
     //    && ksycocaInstance->self == this)
     //    ksycocaInstance->self = 0;
 }
@@ -397,7 +421,8 @@ void KSycocaPrivate::closeDatabase()
         munmap(const_cast<char *>(sycoca_mmap), sycoca_size);
         sycoca_mmap = nullptr;
     }
-    delete m_mmapFile; m_mmapFile = nullptr;
+    delete m_mmapFile;
+    m_mmapFile = nullptr;
 #endif
 
     databaseStatus = DatabaseNotOpen;
@@ -421,12 +446,12 @@ QDataStream *KSycoca::findEntry(int offset, KSycocaType &type)
 {
     QDataStream *str = stream();
     Q_ASSERT(str);
-    //qCDebug(SYCOCA) << QString("KSycoca::_findEntry(offset=%1)").arg(offset,8,16);
+    // qCDebug(SYCOCA) << QString("KSycoca::_findEntry(offset=%1)").arg(offset,8,16);
     str->device()->seek(offset);
     qint32 aType;
     *str >> aType;
     type = KSycocaType(aType);
-    //qCDebug(SYCOCA) << QString("KSycoca::found type %1").arg(aType);
+    // qCDebug(SYCOCA) << QString("KSycoca::found type %1").arg(aType);
     return str;
 }
 
@@ -474,8 +499,7 @@ bool KSycocaPrivate::checkDatabase(BehaviorsIfNotFound ifNotFound)
         // Database exists, and version is ok, we can read it.
 
         if (qAppName() != QLatin1String(KBUILDSYCOCA_EXENAME) && ifNotFound != IfNotFoundDoNothing) {
-
-            // Ensure it's uptodate, rebuild if needed
+            // Ensure it's up-to-date, rebuild if needed
             checkDirectories();
 
             // Don't check again for some time
@@ -512,7 +536,7 @@ QDataStream *KSycoca::findFactory(KSycocaFactoryId id)
         }
         *str >> aOffset;
         if (aId == id) {
-            //qCDebug(SYCOCA) << "KSycoca::findFactory(" << id << ") offset " << aOffset;
+            // qCDebug(SYCOCA) << "KSycoca::findFactory(" << id << ") offset " << aOffset;
             str->device()->seek(aOffset);
             return str;
         }
@@ -544,7 +568,7 @@ KSycocaHeader KSycocaPrivate::readSycocaHeader()
         if (aId) {
             *str >> aOffset;
         } else {
-            break;    // just read 0
+            break; // just read 0
         }
     }
     // We now point to the header
@@ -560,7 +584,7 @@ KSycocaHeader KSycocaPrivate::readSycocaHeader()
     QStringList fileList;
     *str >> fileList;
     extraFiles.clear();
-    for (auto fileName : fileList) {
+    for (const auto &fileName : std::as_const(fileList)) {
         qint64 mtime;
         *str >> mtime;
         extraFiles.insert(fileName, mtime);
@@ -577,26 +601,26 @@ KSycocaHeader KSycocaPrivate::readSycocaHeader()
     return header;
 }
 
-
 class TimestampChecker
 {
 public:
     TimestampChecker()
         : m_now(QDateTime::currentDateTime())
-    {}
+    {
+    }
 
     // Check times of last modification of all directories on which ksycoca depends,
     // If none of them is newer than the mtime we stored for that directory at the
     // last rebuild, this means that there's no need to rebuild ksycoca.
-    bool checkDirectoriesTimestamps(const QMap<QString, qint64> &dirs)
+    bool checkDirectoriesTimestamps(const QMap<QString, qint64> &dirs) const
     {
         Q_ASSERT(!dirs.isEmpty());
-        //qCDebug(SYCOCA) << "checking file timestamps";
+        // qCDebug(SYCOCA) << "checking file timestamps";
         for (auto it = dirs.begin(); it != dirs.end(); ++it) {
             const QString dir = it.key();
             const qint64 lastStamp = it.value();
 
-            auto visitor = [&] (const QFileInfo &fi) {
+            auto visitor = [&](const QFileInfo &fi) {
                 const QDateTime mtime = fi.lastModified();
                 if (mtime.toMSecsSinceEpoch() > lastStamp) {
                     if (mtime > m_now) {
@@ -617,7 +641,7 @@ public:
         return true;
     }
 
-    bool checkFilesTimestamps(const QMap<QString, qint64> &files)
+    bool checkFilesTimestamps(const QMap<QString, qint64> &files) const
     {
         for (auto it = files.begin(); it != files.end(); ++it) {
             const QString fileName = it.key();
@@ -653,14 +677,14 @@ void KSycocaPrivate::checkDirectories()
 bool KSycocaPrivate::needsRebuild()
 {
     if (!timeStamp && databaseStatus == DatabaseOK) {
-        (void) readSycocaHeader();
+        (void)readSycocaHeader();
     }
     // these days timeStamp is really a "bool headerFound", the value itself doesn't matter...
     // KF6: replace it with bool.
-    bool ret = timeStamp != 0 &&
-            (!TimestampChecker().checkDirectoriesTimestamps(allResourceDirs)
-            ||
-            !TimestampChecker().checkFilesTimestamps(extraFiles));
+    const auto timestampChecker = TimestampChecker();
+    bool ret = timeStamp != 0
+        && (!timestampChecker.checkDirectoriesTimestamps(allResourceDirs) //
+            || !timestampChecker.checkFilesTimestamps(extraFiles));
     if (ret) {
         return true;
     }
@@ -692,7 +716,7 @@ bool KSycocaPrivate::buildSycoca()
 quint32 KSycoca::timeStamp()
 {
     if (!d->timeStamp) {
-        (void) d->readSycocaHeader();
+        (void)d->readSycocaHeader();
     }
     return d->timeStamp / 1000; // from ms to s
 }
@@ -702,7 +726,7 @@ quint32 KSycoca::timeStamp()
 quint32 KSycoca::updateSignature()
 {
     if (!d->timeStamp) {
-        (void) d->readSycocaHeader();
+        (void)d->readSycocaHeader();
     }
     return d->updateSig;
 }
@@ -733,7 +757,7 @@ QString KSycoca::absoluteFilePath(DatabaseType type)
 QString KSycoca::language()
 {
     if (d->language.isEmpty()) {
-        (void) d->readSycocaHeader();
+        (void)d->readSycocaHeader();
     }
     return d->language;
 }
@@ -742,7 +766,7 @@ QString KSycoca::language()
 QStringList KSycoca::allResourceDirs()
 {
     if (!d->timeStamp) {
-        (void) d->readSycocaHeader();
+        (void)d->readSycocaHeader();
     }
     return d->allResourceDirs.keys();
 }
@@ -791,8 +815,9 @@ void KSycoca::connectNotify(const QMetaMethod &signal)
 
 void KSycoca::clearCaches()
 {
-    if (ksycocaInstance.exists() && ksycocaInstance()->hasSycoca())
+    if (ksycocaInstance.exists() && ksycocaInstance()->hasSycoca()) {
         ksycocaInstance()->sycoca()->d->closeDatabase();
+    }
 }
 
 extern KSERVICE_EXPORT int ksycoca_ms_between_checks;
